/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};
gc.services = gc.services || {};

var saveText = function(fname, text) {
    var blob = new Blob([text], {
        type: 'text/plain;charset=utf-8'
    });
    saveAs(blob, fname);

    return false;
};

function convertTimestamp(timestamp) {
    var d = new Date(timestamp), // Convert the passed timestamp to milliseconds
        hr = ('0' + d.getHours()).slice(-2),
        min = ('0' + d.getMinutes()).slice(-2), // Add leading 0.
        sec= ('0' + d.getSeconds()).slice(-2), // Add leading 0.
        msec= ('0' + d.getMilliseconds()).slice(-3), // Add leading 0.
        time;
    time = hr + ':' + min + ':' + sec + '.' + msec;
    return time;
}

/*
*  Boilerplate code for creating computed data bindings
*/
document.addEventListener('gc-databind-ready', function() 
{
    /* 
	*   Add custom computed value databindings here, using the following method:
	*
    *   function gc.databind.registry.bind(targetBinding, modelBinding, [getter], [setter]);
	*
	*   param targetBinding - single binding string or expression, or array of binding strings for multi-way binding.
	*   param modelBinding - single binding string or expression, or array of binding strings for multi-way binding.
	*   param getter - (optional) - custom getter function for computing the targetBinding value(s) based on modelBinding value(s).
	*   param setter - (optional) - custom setter function for computing the modelBinding value(s) based on targetBinding value(s).
    */
	
	// For example, a simple computed values based on simple expression
	// gc.databind.registry.bind('widget.id.propertyName', "targetVariable == 1 ? 'binding is one' : 'binding is not one'");
	
	// Or a custom two-way binding with custome getter and setter functions.  (setter is optional)  (getter only indicates one-way binding)
	// gc.databind.registry.bind('widget.id.propertyName', "targetVariable", function(value) { return value*5/9 + 32; }, function(value) { (return value-32)*9/5; });
	
	// Event 1 to n bindings
	/* 
    gc.databind.registry.bind('widget.date.value', 
        // dependant bindings needed in order to compute the date, in name/value pairs.
        {
            weekday: 'widget.dayOfWeek.selectedText',
            day: 'widget.dayOfMonth.value',
            month: 'widget.month.selectedText',
            year: 'widget.year.value'
        }, 
        // getter for date computation
        function(values) 
        {
            // compute and return the string value to bind to the widget with id 'date'
            return values.weekday + ', ' + values.month + ' ' + values.day + ', ' + values.year;
        }
    ); 
	*/
 	// Custome getter for temperature numberbox, for temperature conversion
 	gc.databind.registry.bind(
 	    'widget.ti_widget_numberbox_temp_sens.value',
        'ti_model_streaming.TEMP_SENS',
        function(value) {                       // getter (EVM to widget), dont forget to unlink model from textbox value property!
            switch(ti_widget_droplist_temp_unit.selectedValue) {
                case 'C':
                    return(String(value));
                case 'F':
                    return(String(value * 1.8 + 32));
                case 'K':
                    return(String(273.15 + value));
                default:
                    return(String(value));
            }
        } 
    );
 	// Custome getter for temperature graph, for temperature conversion
 	gc.databind.registry.bind(
 	    'widget.ti_widget_temperaturegraph.temperature',
        'ti_model_streaming.TEMP_SENS',
        function(value) {                       // getter (EVM to widget), dont forget to unlink model from textbox value property!
            switch(ti_widget_droplist_temp_unit.selectedValue) {
                case 'C':
                    return(value);
                case 'F':
                    return(value * 1.8 + 32);
                case 'K':
                    return(273.15 + value);
                default:
                    return(value);
            }
        } 
    );
});

/*
*  Boilerplate code for creating custom actions
*/
document.addEventListener('gc-nav-ready', function() 
{
    /* 
	*   Add custom actions for menu items using the following api:
	*
    *   function gc.nav.registryAction(id, runable, [isAvailable], [isVisible]);
	*
	*   param id - uniquely identifies the action, and should correspond to the action property of the menuaction widget.
	*   param runable - function that performs the custom action.
	*   param isAvailable - (optional) - function called when the menu action is about to appear.  Return false to disable the action, or true to enable it.
	*   param isVisible - (optional) - function called when the menu action is about to appear.  Return false to hide the action, or true to make it visible.
    */
	
	// For example,
	// gc.nav.registerAction('myCustomCloseAction', function() { window.close(); }, function() { return true; }, function() { return true; });
	
	// Alternatively, to programmatically disable a menu action at any time use:
	// gc.nav.disableAction('myCustomCloseAction);    then enable it again using:  gc.nav.enableAction('myCustomCloseAction'); 
});

/*
*  Boilerplate code for working with components in the application gist
*/

var initComplete = false;
var templateObj;

// Wait for DOMContentLoaded event before trying to access the application template
var init = function() {
    templateObj = document.querySelector('#template_obj');

    // Wait for the template to fire a dom-change event to indicate that it has been 'stamped'
    // before trying to access components in the application.
	templateObj.addEventListener('dom-change',function(){
	    if (initComplete) return;
	    this.async(function(){
    	    initComplete = true;
    	    console.log("Application template has been stamped.");
  	        // Now that the template has been stamped, you can use 'automatic node finding' $ syntax to access widgets.
  	        // e.g. to access a widget with an id of 'widget_id' you can use templateObj.$.widgetId
  	        
  	        // Clear chart when chaning temperature scale 
  	        templateObj.$.ti_widget_droplist_temp_unit.addEventListener("change", function(event) {
                templateObj.$.ti_widget_temperaturegraph.reset();
            });

  	        // Open User's Guide link
  	        templateObj.$.ti_widget_button_userguidelink.addEventListener("click", function(event) {
                window.open("https://www.ti.com/lit/pdf/slvubx1");
            });

  	        // Clear chart button
  	        templateObj.$.ti_widget_button_clearchart.addEventListener("click", function(event) {
                templateObj.$.ti_widget_temperaturegraph.reset();
            });
            
            // Save chart data button
            templateObj.$.ti_widget_button_savechart.addEventListener("click", function(event) {
                
                //Print Header information for .csv file to result string
                var result = "";                                               // Label Row for data + Date
                var indx = 0;
                var strData = templateObj.$.ti_widget_temperaturegraph.getDataAsJsonStr();
                
                strData = strData.substr(1);
                while((indx = strData.search("\\[")) != -1) {
                    strData = strData.substr(indx+1);
                    result = result + convertTimestamp(parseInt(strData)) + ",";
                    indx = strData.search("\"");
                    strData = strData.substr(indx+1);
                    result = result + parseFloat(strData) + "\n";
                }
                saveText("tempChart.csv", result);
            });
        },1);
	});
};

templateObj = document.querySelector('#template_obj');
if (templateObj) {
    init();
} else {
    document.addEventListener('DOMContentLoaded',init.bind(this))
}


